#pragma once

/* ******************************************************************************************************************* /
 *
 *	RapidXml Helper - 1.0 (2010-06-16) 
 *
 * ********************************************************************************************************************/

#include <rapidxml/rapidxml.hpp>
#include <atldef.h> //for ATLASSERT
#include <wtypes.h> //for LPCTSTR LPTSTR
#include <tchar.h> //for LPCTSTR LPTSTR
#include <vector>
#include <atlstr.h>
using namespace rapidxml;

typedef xml_node<TCHAR> RxNode;

#define IfFailedReturnFalse(x) if ( ! (x) ) { return FALSE; }
#define IfFailedReturnNull(x) if ( ! (x) ) { return NULL; }
#define IfFailedReturnX(x, ret) if ( ! (x) ) { return ret; }

/**
 *	@brief RxHelper
 */
struct RxHelper
{
	static int	   ReadFileContent(LPCTSTR fileName, OUT char* & pOut);
	static RxNode* AddChild( RxNode* p , LPCTSTR strName, LPCTSTR szValue );
	static BOOL	   RemoveChild( RxNode*p, RxNode* child );
	static RxNode* NodeElement( RxNode* p, LPCTSTR szNodes, BOOL bIfEmptyCreate  );
	static LPCTSTR Text( RxNode* p, LPCTSTR szDefult );
	static LPCTSTR NodeText( RxNode* p, LPCTSTR szNodes, LPCTSTR szDefault );
	static LPCTSTR Attribute(RxNode* p, LPCTSTR szName, LPCTSTR szDefault );
	static LPCTSTR NodeAttribute( RxNode* p, LPCTSTR szNodes, LPCTSTR szName, LPCTSTR szDefault );
	static BOOL	   SetText( RxNode* p, LPCTSTR text);
	static BOOL	   SetAttribute( RxNode*p, LPCTSTR name, LPCTSTR value );
	static void    SetError( rapidxml::parse_error &ex);
	static void    SetError(LPCTSTR szErrorMsg);
	static const int kErrorMsgLength = 512;
	static TCHAR m_szErrorMsg[ kErrorMsgLength ];
};

class CRxEle;
typedef std::vector<CRxEle> CRxEleArray;

/**
 *	@brief CRxEle
 */
class CRxEle
{
public:
	CRxEle( RxNode* node ) : m_XmlNode( node )
	{
	}

	inline RxNode* XmlNode()
	{
		return m_XmlNode;
	}

	inline bool IsError() 
	{ 
		return (m_XmlNode == NULL);
	}

	inline bool IsValid()
	{
		return ! IsError();
	}

	inline CRxEle Parent()
	{
		ATLASSERT( m_XmlNode != NULL);
		return CRxEle(m_XmlNode->parent());
	}

	inline CRxEle Child( LPCTSTR szName = NULL )
	{
		ATLASSERT( m_XmlNode != NULL);
		return CRxEle(m_XmlNode->first_node( szName ));
	}

	inline CRxEle Next ( LPCTSTR szName = NULL )
	{
		ATLASSERT( m_XmlNode != NULL);
		return CRxEle(m_XmlNode->next_sibling( szName ));
	}	

	inline CRxEle Prev( LPCTSTR szName = NULL )
	{
		ATLASSERT( m_XmlNode != NULL);
		return CRxEle(m_XmlNode->previous_sibling( szName ));
	}

	inline int Type()
	{
		ATLASSERT( m_XmlNode != NULL);
		return m_XmlNode->type();
	}

	inline LPCTSTR Name()
	{
		ATLASSERT( m_XmlNode != NULL);
		return m_XmlNode->name();
	}

	inline CRxEle NodeElement( LPCTSTR szNodes )
	{
		ATLASSERT( m_XmlNode != NULL);
		return CRxEle(RxHelper::NodeElement( m_XmlNode, szNodes, FALSE));
	}

	inline LPCTSTR Text( LPCTSTR szDefault = _T("") )
	{
		ATLASSERT( m_XmlNode != NULL);
		return RxHelper::Text( m_XmlNode, szDefault);
	}

	inline LPCTSTR NodeText( LPCTSTR szNodes, LPCTSTR szDefault = _T("") )
	{
		ATLASSERT( m_XmlNode != NULL);
		return RxHelper::NodeText( m_XmlNode, szNodes, szDefault );
	}

	inline LPCTSTR Attribute( LPCTSTR szName, LPCTSTR szDefault = _T("") )
	{
		ATLASSERT( m_XmlNode != NULL);
		return RxHelper::Attribute( m_XmlNode, szName, szDefault );
	}

	inline LPCTSTR NodeAttribute( LPCTSTR szNodes, LPCTSTR szName, LPCTSTR szDefault = _T(""))
	{
		ATLASSERT( m_XmlNode != NULL);
		return RxHelper::NodeAttribute( m_XmlNode, szNodes, szName, szDefault );
	}

	inline int TextByInt( int nDef = 0 )
	{
		return ToInt( Text( NULL ), nDef);
	}

	inline int NodeTextByInt( LPCTSTR szNodes, int nDef = 0 )
	{
		ATLASSERT( m_XmlNode != NULL);
		return ToInt( RxHelper::NodeText( m_XmlNode, szNodes, NULL ), nDef ) ;
	}

	inline int AttributeByInt( LPCTSTR szName, int nDef = 0 )
	{
		ATLASSERT( m_XmlNode != NULL);
		return ToInt( RxHelper::Attribute( m_XmlNode, szName, NULL ), nDef );
	}

	inline int NodeAttributeByInt( LPCTSTR szNodes, LPCTSTR szName, int nDef = 0 )
	{
		ATLASSERT( m_XmlNode != NULL);
		return ToInt( RxHelper::NodeAttribute( m_XmlNode, szNodes, szName, NULL ), nDef );
	}

	inline operator bool() 
	{ 
		return IsValid() ;
	}

	inline operator RxNode* ()
	{
		return XmlNode();
	}

	int		SelectNodes( CRxEleArray& ar, LPCTSTR szNodes = NULL, LPCTSTR szNameFilter = NULL )
	{
		ATLASSERT( m_XmlNode != NULL);
		RxNode* curr_node = m_XmlNode;
		if ( szNodes )
		{
			curr_node = RxHelper::NodeElement( curr_node,  szNodes, FALSE );
			IfFailedReturnX( curr_node, 0);
		}
		RxNode* node = curr_node->first_node( szNameFilter );
		IfFailedReturnX( node, 0);
		do 
		{
			if ( node->type() == node_element )
			{
				ar.push_back( CRxEle(node) );
			}
			node = node->next_sibling( szNameFilter );
		} 
		while ( node != NULL );
		return (int)ar.size();
	}

	inline LPCTSTR LastErrorMsg()
	{
		return RxHelper::m_szErrorMsg;
	}

	inline BOOL	SetText(LPCTSTR text)
	{
		ATLASSERT( m_XmlNode != NULL);
		return RxHelper::SetText( m_XmlNode, text);
	}
	
	inline BOOL	SetAttribute(LPCTSTR name, LPCTSTR text)
	{
		ATLASSERT(m_XmlNode != NULL);
		return RxHelper::SetAttribute( m_XmlNode, name, text);
	}
	
	inline CRxEle	AddChild( LPCTSTR name, LPCTSTR szValue = NULL)
	{
		ATLASSERT( m_XmlNode != NULL);
		return CRxEle( RxHelper::AddChild( m_XmlNode, name, szValue));
	}
	
	inline BOOL	RemoveChild( CRxEle& child )
	{
		ATLASSERT(m_XmlNode != NULL);
		ATLASSERT(child.IsValid());
		return RxHelper::RemoveChild(m_XmlNode, child.XmlNode());
	}

protected:
	inline int ToInt( LPCTSTR p, int& nDef)
	{
		IfFailedReturnX(p, nDef);
		return _ttoi(p);
	}

	RxNode  *m_XmlNode;
};


/**
 *	@brief CRxDoc
 */
class CRxDoc : public CRxEle
{
public:
	CRxDoc() : m_bOK ( FALSE ) , CRxEle(&m_XmlDoc)
	{
	}
	
	BOOL	LoadFile(LPCTSTR szPath);	///< XML  ε
	
	BOOL	ParseString(LPCTSTR xmlString);
	
	BOOL	IsError()
	{
		return !m_bOK;
	}
	
	BOOL	IsValid()
	{
		return m_bOK;
	}

	BOOL	LoadURL(LPCTSTR szURL);

	/* ******************************************************************************************************************* /
	 *
	 *	Write Operation
	 *
	 * ********************************************************************************************************************/	
	BOOL	NewDocument( LPCTSTR startElement );
	
	CRxEle	Root();
	
	CRxEle	Declaration();
	
	BOOL	SaveFile( LPCTSTR fileName );

protected:
	BOOL m_bOK;
	xml_document<TCHAR>  m_XmlDoc;
};
